package base;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Collection;
import java.util.Iterator;
import java.util.TreeSet;

import base.CLI.CLIMenu;

public class NetworkMenu implements IMenu {
	
	private String myMenuID;
	private Collection<Method> methods;

	public NetworkMenu(String menuID, Method[] menuItems) {
		myMenuID = menuID;
		methods = new TreeSet<Method>(new MethodComparator());
		CLIMenu ann = null;
  	for (int i = 0; i < menuItems.length; i++) {
			Method method = menuItems[i];
			ann = method.getAnnotation(CLIMenu.class);
			if (ann != null) {
				if (ann.menuID().matches(".*"+myMenuID+".*")) {
					methods.add(method);
				}
			}
		}
  	printMenu();
	}
	
	public NetworkMenu(Method[] menuItems) {
		this("Network", menuItems);
	}
	
	/* (non-Javadoc)
	 * @see base.IMenu#parseArg()
	 */
	@Override
	public Method parseArg() {
		int swValue;
    for (;;) { // main loop, do this forever, or, until someone calls leave()
    	swValue = Keyin.inInt("Enter option index (99 for help): ");
    	return ((Method) methods.toArray()[swValue]);
    } 
	}
	
	public void printMenu() {

    // Display menu graphics to inform the user of what level they are on
    System.out.println("==================================");
    System.out.printf( "\t%s Menu%n",myMenuID);
    System.out.println("==================================");
		int index = 0;
		// go through our methods looking for the appropriate annotations 
		for (Iterator<Method> iterator = methods.iterator(); iterator.hasNext();) {
			Method method = iterator.next();
			CLIMenu ann = method.getAnnotation(CLIMenu.class);
				// then pretty print
				System.out.printf("\t%2d. %s%n",index++,ann.menuText());
		}
	}
	
	public String getState() {
		return myMenuID;
	}
	
}
