package base;

import java.io.Serializable;
import java.text.DateFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Date;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.UUID;

import org.chronicj.DateRange;
import org.chronicj.Schedule;
import org.chronicj.TemporalExpression;
import org.chronicj.TimePoint;
import org.chronicj.impl.ArbitraryDateRangeTE;
import org.chronicj.impl.DayInMonthTE;
import org.chronicj.impl.DefaultSchedule;
import org.chronicj.impl.DifferenceTE;
import org.chronicj.impl.IntersectionTE;
import org.chronicj.impl.RangeEachDayTE;
import org.chronicj.impl.RangeEachYearTE;
import org.chronicj.impl.UnionTE;

/**
 * Class Task
 */
public class Task implements Serializable, org.chronicj.Event {

  //
  // Fields
  //

  private String schedule = "";
  private int expValue = 0;
  private int numPayouts = 1; //TODO: payouts should be turned into subtasks, 
                              //should be 'get total value' and 'get value increments' methods
  private String[] dateFormat = {"dd","/MM","/yyyy"};
  private String description = "";
  private String name = "";
  private Collection<String> executers;
  private Schedule mySchedule;
  
  private UUID myID;
  

  //
  // Constructors
  //
  public Task () { 

		myID = UUID.randomUUID();
    executers = new ArrayList<String>();
    mySchedule = new DefaultSchedule();
  };

  //
  // Methods
  //


  //
  // Accessor methods
  //

  /**
   * Set the value of expValue
   * @param newVar the new value of expValue
   */
  public void setExpValue ( int expVal ) {
    expValue = expVal;
  }

  /**
   * Get the value of expValue
   * @return the value of expValue
   */
  public int getExpValue ( ) {
    return expValue;
  }
  
  /**
   * Get a list of values of all the sub tasks
   * @return the value of the sub tasks
   */
  public int[] getExpValueIncrements( ) {
    return new int[]{expValue};
  }

  /**
   * Set the value of numPayouts
   * @param newVar the new value of numPayouts
   */
  public void setNumPayouts ( int numPayouts ) {
    this.numPayouts = numPayouts;
  }

  /**
   * Get the value of numPayouts
   * @return the value of numPayouts
   */
  public int getNumPayouts ( ) {
    return numPayouts;
  }

  //
  // Other methods
  //


  /**
   * @return       int
   * @param        executerName
   */
  public int execute( String executerName )
  {
    if (!(executers.size()==getNumPayouts())) {
      executers.add(executerName);
      return expValue/getNumPayouts();
    } else {
      return 0;
    }
  }
  
  public Collection<String> getExecuters(){
    return Collections.unmodifiableCollection(executers);
  }
  
  /**
   * 
   * @return the percent that the task is completed
   */
  public int percentDone() {
    return (100*executers.size())/getNumPayouts();
  }

  /**
   * @param description the description to set
   */
  public void setDescription(String description) {
    this.description = description;
  }

  /**
   * @return the description
   */
  public String getDescription() {
    return description;
  }

  /**
   * @param name the name to set
   */
  public void setName(String name) {
    this.name = name;
  }

  /**
   * @return the name
   */
  public String getName() {
    return name;
  }

	@Override
	public String getId() {
		return myID.toString();
	}
	
	///Scheduling Related Methods///
	//TODO:Make this work with temporal expressions et cetera
	//!:Make this work with temporal expressions et cetera
  /**
   * Set the value of schedule
   * The schedule is a string contains the following arguments:
   * 	date	(dd[/mm[/yyyy]])
   * 	time	(hh[:]mm) [ ((a.m.|am) | (p.m.|pm)) ]
   * 	arg		+ multiplier ((min,minutes)| (d|days) | (w|weeks) | (m, months))
   * 
   * Acceptable syntax is:
   * 	(time [date] | date [time]) arg* 
   * 
   * Needs to have a termination date for a repeating pattern, probably a 
   * second date that it repeats to, rather than a repeat num time thing.
   * @param regexp the new value of schedule
   */
  public void setSchedule ( String regexp ) {
    schedule = regexp;
    if (regexp.matches(".*-.*")) {
	    ExpressionBuilder eb = new ExpressionBuilder();
	    mySchedule.add(this, eb.parse(regexp));
    }
  }
  
  /**FIXME: This should use the interperater pattern, currently doesn't support 
   * 'repeat until x date' type things.
   * FIXME: Schedule should be a seperate object, BAM!
   * @return       List<java.util.Date>
   */
  public Collection<Date> getSchedule(  )
  {
    return generateDatesFromSchedule(12);
  }
  
  public Collection<Date> getSchedule(DateRange during) {
  	return null;
  }

  /**
   * @return       List<java.util.Date>
   */
  public Iterator<Date> getScheduleIt(  )
  {
    return generateDatesFromSchedule(12).iterator();
  }

  private Collection<java.util.Date> generateDatesFromSchedule(int numMonths) {
    //--- actual parsing of the string  ----//
    String[] pattern = schedule.toLowerCase().split(" ");
    SimpleDateFormat sdf = new SimpleDateFormat();
    sdf.setLenient(false); // we get unexpected results from unexpected input
    String tempDateFormat = "";
    int endOfDatePattern = 0;
    Map<String,Integer> multipliers = new HashMap<String,Integer>();
    for (int i = 0;i<pattern.length;i++) {
      // first we look for either a date, or a time
      if ((pattern[i].length() <= 2 && !(pattern[i].contains("+")||pattern[i].contains("a")||pattern[i].contains("p"))) || pattern[i].contains("/")) {
        // we have a date: date	= (dd[/mm[/yyyy]])
        String[] dateFields = pattern[i].split("/");
        for (int j=0;j<dateFields.length;j++) {

          tempDateFormat  += dateFormat[j];
        }
        endOfDatePattern = i;
      } else if (pattern[i].length() == 4 || pattern[i].contains(":")) { // hopefully we have a time
        // yup. we have a time: time = (hh[:]mm) [ ((a.m.|am) | (p.m.|pm)) ]
        if (pattern[i].charAt(0) > '1' || pattern[i].charAt(1) > '2') {
          // must be in 24 hour time
          tempDateFormat += "HH";
        } else {
          tempDateFormat += "hh";
        }
        if (pattern[i].charAt(2) == 58) {
          tempDateFormat += ":";
        }
        tempDateFormat += "mm";

        // now we need to see if the next field contains am/pm codes
        if (pattern[i].contains("a")||pattern[i].contains("p")) {
          tempDateFormat += "a";
        } else if ((i+1)<pattern.length&&
            (pattern[i+1].contains("a")||pattern[i+1].contains("p"))) {
          tempDateFormat += "a";
          ++i;
        }
        endOfDatePattern = i;
      } else if (pattern[i].contains("+")) {
        // we have an 'arg': arg = + multiplier ((d|days) | (min,minutes) | (m, months))
        int endOfMultiplier = 2;
        int currMultiplier = 0;
        if (pattern[i].length() > 1) { // the multiplier field is in the first block with the '+', leave index after mult.
          for (;i<pattern.length && pattern[i].charAt(endOfMultiplier) < '9' && 
          pattern[i].charAt(endOfMultiplier) > '0';i++);
          currMultiplier = Integer.parseInt(pattern[i].substring(1,endOfMultiplier));

        } else if ((i+1)<pattern.length&&(pattern[i+1].contains("d")||pattern[i+1].contains("m")||pattern[i+1].contains("w"))) {// the multiplier field is in the next block with the type field 
          // we have both the multiplier and type fields in this block
          ++i;
          for (;pattern[i].charAt(endOfMultiplier) < '9' && 
          pattern[i].charAt(endOfMultiplier) > '0';i++);
          currMultiplier = Integer.parseInt(pattern[i].substring(1,endOfMultiplier));
        } else { // the multiplier field is in the next block and the type is in the next
          ++i;
          currMultiplier= Integer.parseInt(pattern[i]);
          endOfMultiplier=0;
          ++i;
        }

        //-- Sanitise type field --//
        if (pattern[i].charAt(endOfMultiplier) == 'd') {
          multipliers.put("day", currMultiplier);
        } else if (pattern[i].substring(endOfMultiplier).contains("min")) {
          multipliers.put("minute", currMultiplier);
        } else if (pattern[i].charAt(endOfMultiplier) == 'm') { //must be months
          multipliers.put("month", currMultiplier);
        } else if (pattern[i].charAt(endOfMultiplier) == 'w') { // weeks
          multipliers.put("week", currMultiplier);
        }

      }

      //tempDateFormat += " ";
    }
    sdf.applyPattern(tempDateFormat);
    //--- actual parsing of the string ----//

    Date seedDate = new Date(), genUpToDate = new Date();
    Set<Date> returnList = new TreeSet<Date>();
    //---- generate Dates ---//
    String dateString = "";
    for (int i =0;i<endOfDatePattern+1;i++) {
      dateString += pattern[i];
    }
    try { 
      seedDate = sdf.parse(dateString);
    } catch (ParseException e) {
      System.err.printf("Error parsing date from input - invalid date format: %s\n" +
          "Acceptable formats are of the form: %s\n",schedule, "(dd[/mm[/yyyy]])");

      //e.printStackTrace();
    }
    // if the date is the epoch plus some time less than a day, it should actually be relative to today
    if (seedDate.before(new Date(86400000))) {
      long offset = seedDate.getTime();
      seedDate.setTime(System.currentTimeMillis());
      seedDate.setHours(0);
      seedDate.setMinutes(0);
      seedDate.setSeconds(0);
      seedDate.setTime(seedDate.getTime()+offset);
    }
    java.util.GregorianCalendar cal = new java.util.GregorianCalendar();
    //System.out.printf("Available TZs: %s%n",TimeZone.getAvailableIDs());
    Iterator<String> it = multipliers.keySet().iterator();
    String literal = "";
    cal.setTime(seedDate);
    returnList.add(cal.getTime());
    int mult = 0;
    while(it.hasNext()) {
      // apply each multiplier until the passed since the original date is numMonths
      literal = it.next();
      cal.add(Calendar.MONTH, numMonths);
      genUpToDate = cal.getTime(); // 
      cal.add(Calendar.MONTH, -numMonths);
      mult = multipliers.get(literal);
      // Add dates matching each pattern for a year into the future
      while (genUpToDate.after(cal.getTime())) {
        if (literal.toLowerCase().charAt(0) == 'd') {
          cal.add(Calendar.DAY_OF_MONTH, mult);
        } else if (literal.toLowerCase().contains("i")) {//must be minutes
          cal.add(Calendar.MINUTE, mult);
        } else if(literal.toLowerCase().charAt(0) == 'm') {//must be months
          cal.add(Calendar.MONTH, mult);
        } else if(literal.toLowerCase().charAt(0) == 'w') {//must be weeks
          cal.add(Calendar.DATE, mult*7);
        } 
        returnList.add(cal.getTime());
      }
      // set the calender back to where we started
      cal.setTime(seedDate);
    }
    return returnList;
  }

  

}

class ExpressionBuilder {
	
	TemporalExpression te = null;
	ExpressionBuilder() {
		
	}
	
	public TemporalExpression getTE() {
		// TODO Auto-generated method stub
		return null;
	}

	public TemporalExpression parse(String regexp) {
		String[] switches = {"+d","+w","+m","+y"};
		UnionTE ret = new UnionTE();
		TimePoint range = null;
		String[] tokens = regexp.split(" ");
		if (tokens[0].length()==10) {
			//date
			String[] fields = tokens[0].split("-");
			range = new TimePoint(Integer.parseInt(fields[0]), Integer.parseInt(fields[1]), Integer.parseInt(fields[2]));
			ret.add(new ArbitraryDateRangeTE(new DateRange(range, range)));
		} else if (tokens[0].length()==16) {
			//dateTime YYYY-MM-DDThh:mm
			String[] fields = tokens[0].split("[-T:]");
			range = new TimePoint(Integer.parseInt(fields[0]), Integer.parseInt(fields[1]), Integer.parseInt(fields[2]),Integer.parseInt(fields[3]),Integer.parseInt(fields[4]));
			te = new ArbitraryDateRangeTE(new DateRange(range, range));
		} else {
			return null;
		}
    for (int i = 1; i < tokens.length; i++) {
			String token = tokens[i];
			if (token.length()==2) {
				//recurrence specifier
				if (token.equals(switches[0])) {
					ret.add(new RangeEachDayTE(range.getHourOfDay(),range.getMinute(), range.getHourOfDay(),range.getMinute()));
				} else if (token.equals(switches[1])) {
					ret.add(new DayInMonthTE(range.getDayOfWeek()));
				} else if (token.equals(switches[2])) {
					ret.add(new DayInMonthTE(range.getDayOfWeek(),((range.getDayOfMonth() - 1) / 7) + 1));
				} else if (token.equals(switches[3])) {
					te = new IntersectionTE();
					((IntersectionTE) te).add(new RangeEachYearTE(range.getMonth()));
					((IntersectionTE) te).add(new DayInMonthTE(range.getDayOfWeek(),((range.getDayOfMonth() - 1) / 7) + 1));
					ret.add(te);
				}
			}
		}
		return te;
	}
}