package base;

import static org.junit.Assert.*;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.chronicj.DateRange;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class TaskTest {

	Task test01;
	Task test02;
	Task test03;
	
	@Before
	public void setUp() throws Exception {
		test01 = new Task();
		test02 = new Task();
		test03 = new Task();
	}

	@After
	public void tearDown() throws Exception {
	}

	@Test
	public void testTask() {
	}
	
	@Test
	public void testGetExpValue ( ) {
		assertEquals(test01.getExpValue(),0);
		test01.setExpValue(999);
		assertEquals(test01.getExpValue(),999);
	  }
	
	@Test
	public void testGetNumPayouts ( ) {
		assertEquals(test01.getNumPayouts(),1);
		test01.setNumPayouts(3);
		assertEquals(test01.getNumPayouts(),3);
	  }
	/**
	   * Set the value of schedule
	   * The schedule is a string contains the following arguments:
	   * 	date	(dd[/mm[/yyyy]])
	   * 	time	(hh[:]mm) [ ((a.m.|am) | (p.m.|pm)) ]
	   * 	arg		+ multiplier ((d|days) | (min,minutes) | (m, months))
	   * 
	   * Acceptable syntax is:
	   * 	(time [date] | date [time]) arg*
	   * */
	@Test
	public void testGetSchedule() {
		
		Date tD = new Date();
		List<Date> tL = new ArrayList<Date>();
		SimpleDateFormat sdf = new SimpleDateFormat();
		
		/*Things to test 
		 * Non repeating (single datetime)
		 * 	td = YYYY-DD-MMTHH:mm
		 * Day of the week (using DayInMonth (union))
		 * 	By date(s), by number(s), by range of both
		 * 		td +dw(mon,tue)
		 * 		td +dw(1,2)  
		 * 		td +d(2,5) repeat every second day from td
		 * 		td +dw(mon-tue)
		 * 		td +dw(1-2)
		 * Day of month
		 * 	As above, but specifying weeks
		 * 		td +dm(6,-1) //last friday
		 * 		td +dm(3,2) // second tuesday
		 * Month of year (using monthly recurrence intersect with REY(m))
		 * 	Using names, using dates, using range
		 * 		td +m(1)
		 * 		td +m(2,5)
		 * 		td +m(feb,april)
		 * Repeat yearly (birthdays etc)
		 * 	Probably using dayinmonth
		 * 		as above
		 * 
		 * 
		 * Actually, just use
		 * td +d //repeat every day
		 * td +m //repeat every month
		 * td +y //repeat each year
		 */
		
		String td = "2010-02-10";
		String t1 = "+d";
		String t2 = "+w";
		String t3 = "+m";
		String t4 = "+y";

		sdf.applyPattern("dd/MM/yyyy");
		//test the non repeating case
		test01.setSchedule(td);
		try {
			assertEquals(sdf.parse("02/10/2010"),test01.getSchedule().toArray()[0]);
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		
	//repeat every day
		try {
		tL.add(sdf.parse("02/10/2010"));//+0*3month
		tL.add(sdf.parse("03/10/2010"));//+1*3month
		tL.add(sdf.parse("04/10/2010"));//+2*3month
		tL.add(sdf.parse("05/10/2010"));//+3*3month
		assert(test01.getSchedule().isEmpty());
		test01.setSchedule(td+t1);
		Date[] gd = (test02.getSchedule(new DateRange(sdf.parse("01/10/2010"), sdf.parse("05/10/2010")))).toArray(new Date[] {});int i=0;
//		System.out.printf("Generated Array Length = %d%n",gd.length);
//		for (Date date : tL) {
//			System.out.printf("testDate: %s -- generatedDate: %s%n",date,gd[i++]);
//		}i=0;
		for (Date date : tL) {
			assertEquals(gd[i++].toString(),date.toString());
		}
		
	//repeat every month
		sdf.applyPattern("dd/MM/yyyy");
		tL.add(sdf.parse("02/11/2010"));//+0*3month
		tL.add(sdf.parse("02/12/2010"));//+1*3month
		tL.add(sdf.parse("02/01/2011"));//+2*3month
		tL.add(sdf.parse("02/02/2011"));//+3*3month
		assert(test01.getSchedule().isEmpty());
		test01.setSchedule(td+t1);
		gd = (test02.getSchedule(new DateRange(sdf.parse("01/10/2010"), sdf.parse("05/10/2010")))).toArray(new Date[] {});i=0;
//		System.out.printf("Generated Array Length = %d%n",gd.length);
//		for (Date date : tL) {
//			System.out.printf("testDate: %s -- generatedDate: %s%n",date,gd[i++]);
//		}i=0;
		for (Date date : tL) {
			assertEquals(gd[i++].toString(),date.toString());
		}
		} catch (ParseException e1) {
			e1.printStackTrace();
		}
		//--- Testing date and time functionality ---//
//		try {
//			sdf.applyPattern("dd/MM/yyyy");
//			assert(test01.getSchedule().isEmpty());
//			sdf.applyPattern("dd/MM/yyyy");
//			test01.setSchedule("25/02/1989");
//			tD = sdf.parse("25/02/1989");
//			assertEquals(tD,test01.getSchedule().toArray()[0]);
//			
//			test01.setSchedule("25/02/1989 0220"); // good time and date
//			sdf.applyPattern("dd/MM/yyyy HHmm");
//			tD = sdf.parse("25/02/1989 0220");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//			
//			test01.setSchedule("25/02/1989 2020"); // 24 hour time
//			sdf.applyPattern("dd/MM/yyyy HHmm");
//			tD = sdf.parse("25/02/1989 2020");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//			
//			test01.setSchedule("25/02/1989 02:20"); // colon
//			sdf.applyPattern("dd/MM/yyyy HH:mm");
//			tD = sdf.parse("25/02/1989 02:20");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//			
//			test01.setSchedule("25/02/1989 0220 am"); // a.m.
//			sdf.applyPattern("dd/MM/yyyy hhmm a");
//			tD = sdf.parse("25/02/1989 0220 am");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//			
//			test01.setSchedule("25/02/1989 0220 pm");
//			sdf.applyPattern("dd/MM/yyyy hhmm a");
//			tD = sdf.parse("25/02/1989 0220 pm");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//			
//			test01.setSchedule("25/02/1989 0220 am"); //am
//			sdf.applyPattern("dd/MM/yyyy hhmm a");
//			tD = sdf.parse("25/02/1989 0220 am");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//		} catch (ParseException e) {
//			e.printStackTrace();
//			fail("Parse Exception");
//		}
//		try {
//			test01.setSchedule("25/02/1989 0288"); // bad time (maybe set calendar to lenient (are you even parsing the time?))
//			sdf.applyPattern("dd/MM/yyyy HHmm");
//			tD = sdf.parse("25/02/1989 0288");
//			assertNotSame(test01.getSchedule().toArray()[0],tD);
//		} catch (ParseException e) {
//			e.printStackTrace();
//			fail("Parse Exception");
//		}
//		try {
//			test01.setSchedule("25"); //25/07/2010
//			sdf.applyPattern("dd");
//			tD = sdf.parse("25");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//		} catch (ParseException e) {
//			e.printStackTrace();
//			fail("Parse Exception");
//		}
//		try {
//			test01.setSchedule("25/02"); //25/02/2010
//			sdf.applyPattern("dd/MM");
//			tD = sdf.parse("25/02");
//			assertEquals(test01.getSchedule().toArray()[0],tD);
//		} catch (ParseException e) {
//			e.printStackTrace();
//			fail("Parse Exception");
//		}
//		try {
//			test01.setSchedule("02/25"); // bad date
//			sdf.applyPattern("dd/MM");
//			tD = sdf.parse("25/02");
//			assertNotSame(test01.getSchedule().toArray()[0],tD);
//		} catch (ParseException e) {
//			e.printStackTrace();
//			fail("Parse Exception");
//		}
		//--- End testing date and time functionality ---//
		
		//--- Testing multiplier functionality functionality ---//
//		try {
//			sdf.applyPattern("dd/MM/yyyy");
//			tL.add(sdf.parse("25/02/1989"));//+0*3month
//			tL.add(sdf.parse("25/05/1989"));//+1*3month
//			tL.add(sdf.parse("25/08/1989"));//+2*3month
//			tL.add(sdf.parse("25/11/1989"));//+3*3month
//			assert(test02.getSchedule().isEmpty());
//			test02.setSchedule("25/02/1989 +3months");
//			Date[] gd = (test02.getSchedule()).toArray(new Date[] {});int i=0;
////			System.out.printf("Generated Array Length = %d%n",gd.length);
////			for (Date date : tL) {
////				System.out.printf("testDate: %s -- generatedDate: %s%n",date,gd[i++]);
////			}i=0;
//			for (Date date : tL) {
//				assertEquals(gd[i++].toString(),date.toString());
//			}
//			//System.out.println("--------------\n");
//			tL.clear();
//			sdf.applyPattern("dd/MM/yyyy");
//			tL.add(sdf.parse("25/02/1989"));//+0*3week
//			tL.add(sdf.parse("18/03/1989"));//+1*3week
//			tL.add(sdf.parse("8/04/1989"));//+2*3week
//			tL.add(sdf.parse("29/04/1989"));//+3*3week
//			assert(test02.getSchedule().isEmpty());
//			test02.setSchedule("25/02/1989 +3weeks");
//			gd = (test02.getSchedule()).toArray(new Date[] {});i=0;
////			System.out.printf("Generated Array Length = %d%n",gd.length);
////			for (Date date : tL) {
////				System.out.printf("testDate: %s -- generatedDate: %s%n",date,gd[i++]);
////			}i=0;
//			for (Date date : tL) {
//				assertEquals(gd[i++].toString(),date.toString());
//			}
//			
//			tL.clear();
//			sdf.applyPattern("dd/MM/yyyy");
//			tL.add(sdf.parse("25/02/1989"));//+0*3week
//			tL.add(sdf.parse("18/03/1989"));//+1*3week
//			tL.add(sdf.parse("8/04/1989"));//+2*3week
//			tL.add(sdf.parse("29/04/1989"));//+3*3week
//			assert(test02.getSchedule().isEmpty());
//			test02.setSchedule("25/02/1989 + 3 weeks");
//			gd = (test02.getSchedule()).toArray(new Date[] {});i=0;
//			for (Date date : tL) {
//				assertEquals(gd[i++].toString(),date.toString());
//			}
//
//			tL.clear();
//			sdf.applyPattern("dd/MM/yyyy");
//			tL.add(sdf.parse("25/02/1989"));//+0*3week
//			tL.add(sdf.parse("18/03/1989"));//+1*3week
//			tL.add(sdf.parse("8/04/1989"));//+2*3week
//			tL.add(sdf.parse("29/04/1989"));//+3*3week
//			assert(test02.getSchedule().isEmpty());
//			test02.setSchedule("25/02/1989 + 3 w");
//			gd = (test02.getSchedule()).toArray(new Date[] {});i=0;
//			for (Date date : tL) {
//				assertEquals(gd[i++].toString(),date.toString());
//			}
//			
//			tL.clear();
//			sdf.applyPattern("dd/MM/yyyy");
//			tL.add(sdf.parse("25/02/1989"));//+0*3days
//			tL.add(sdf.parse("28/02/1989"));//+1*3days
//			tL.add(sdf.parse("3/03/1989"));//+2*3days
//			tL.add(sdf.parse("06/03/1989"));//+3*3days
//			assert(test02.getSchedule().isEmpty());
//			test02.setSchedule("25/02/1989 + 3 d");
//			gd = (test02.getSchedule()).toArray(new Date[] {});i=0;
//			for (Date date : tL) {
//				assertEquals(gd[i++].toString(),date.toString());
//			}
//			
//			tL.clear();
//			sdf.applyPattern("dd/MM/yyyy");
//			tL.add(sdf.parse("25/02/1989"));//+0*3days
//			//tL.add(sdf.parse("25/02/1989"));//+0*3week
//			tL.add(sdf.parse("28/02/1989"));//+1*3days
//			tL.add(sdf.parse("3/03/1989"));//+2*3days
//			tL.add(sdf.parse("06/03/1989"));//+3*3days
//			tL.add(sdf.parse("09/03/1989"));//+4*3days
//			tL.add(sdf.parse("12/03/1989"));//+5*3days
//			tL.add(sdf.parse("15/03/1989"));//+6*3days
//			tL.add(sdf.parse("18/03/1989"));//+7*3days
//			//tL.add(sdf.parse("18/03/1989"));//+1*3week
//			//tL.add(sdf.parse("8/04/1989"));//+2*3week
//			//tL.add(sdf.parse("29/04/1989"));//+3*3week
//			assert(test02.getSchedule().isEmpty());
//			test02.setSchedule("25/02/1989 + 3 d + 3 w");
//			gd = (test02.getSchedule()).toArray(new Date[] {});i=0;
//			for (Date date : tL) {
//				System.out.printf("TestDate: %s\tGen Date: %s%n",date.toString(),gd[i++].toString());
//			}
//			for (Date date : tL) {
//				assertEquals(gd[i++].toString(),date.toString());
//			}
//		} catch (ParseException e) {
//			e.printStackTrace();
//			fail("Parse Exception");
//		}
		//--- End testing multiplier functionality functionality ---//
	}

	@Test
	public void testExecute() {
		test03.setExpValue(999);
		test03.setNumPayouts(3);
		assert(test03.execute("HayHayHay")==999/3);
	}

}
