/*
 * ==================================================================== The
 * Apache Software License, Version 1.1
 * 
 * Copyright (c) 2003 Digital Clash LLC. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *  3. The end-user documentation included with the redistribution, if any,
 * must include the following acknowledgment: "This product includes software
 * developed by the ChronicJ team (http://www.chronicj.org/)." Alternately,
 * this acknowledgment may appear in the software itself, if and wherever such
 * third-party acknowledgments normally appear.
 *  4. The names "ChronicJ" and "Digital Clash" not be used to endorse or
 * promote products derived from this software without prior written
 * permission. For written permission, please contact info@digitalclash.com.
 *  5. Products derived from this software may not be called "ChronicJ",
 * "Digital Clash", nor may "ChronicJ" or "Digital Clash" appear in their name,
 * without prior written permission of Digital Clash LLC.
 * 
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * APACHE SOFTWARE FOUNDATION OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ==================================================================== This
 * product includes software developed by the by the Apache Software Foundation
 * (http://www.apache.org/).
 * ====================================================================
 */
package org.chronicj.impl;

import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;

/**
 * Silly helper class for date formatting and other random utilities that don't
 * squarely belong in other classes.
 * 
 * @author <a href="mlipper@US-ABP.com">Matthew Lipper</a>
 * 
 * @see java.text.SimpleDateFormat
 */
public class DateUtil
{
	private static final String DEFAULT_PATTERN =
		"EEE, d MMM yyyy h:mm:ss.S a Z";
	private static final SimpleDateFormat sdf =
		new SimpleDateFormat(DEFAULT_PATTERN);

	/**
	 * Date to String using the default pattern.
	 * 
	 * @param aDate
	 *                    java.util.Date the date
	 * 
	 * @return String formatted with pattern
	 * 
	 * @see java.text.SimpleDateFormat
	 */
	public static String format(Date aDate)
	{
		return format(aDate, DEFAULT_PATTERN);
	}

	/**
	 * Date to String using the supplied pattern.
	 * 
	 * @param aDate
	 *                    java.util.Date the date
	 * @param pattern
	 *                    String the pattern
	 * 
	 * @return String formatted with pattern
	 * 
	 * @see java.text.SimpleDateFormat
	 */
	public static String format(Date aDate, String pattern)
	{
		sdf.applyPattern(pattern);

		return sdf.format(aDate);
	}

	/**
	 * Date format function using supplied pattern.
	 * 
	 * @param dateString
	 *                    String the date
	 * @param pattern
	 *                    String the pattern
	 * 
	 * @return java.util.Date
	 * 
	 * @throws IllegalArgumentException
	 *                    if supplied dateString or pattern are invalid.
	 * 
	 * @see java.text.SimpleDateFormat
	 */
	public static Date parse(String dateString, String pattern)
	{
		try
		{
			sdf.applyPattern(pattern);

			return sdf.parse(dateString);
		} catch (ParseException e)
		{
			throw new IllegalArgumentException(e.getMessage());
		}
	}

	/**
	 * Date format function using default pattern.
	 * 
	 * @param dateString
	 *                    String the date
	 * 
	 * @return java.util.Date
	 * 
	 * @see java.text.SimpleDateFormat
	 */
	public static Date parse(String dateString)
	{
		return parse(dateString, DEFAULT_PATTERN);
	}
}
